package client;

import java.util.*;

import composite.Component;
import composite.Composite;
import composite.Part;
import visitors.*;

public class ComputerBuilderApp {

	public static void main(String[] args) {

		Component computer = new Composite("Computer");
		
		Component systemUnit = new Composite("System Unit"); 
		Component monitor = new Part("Monitor", 200.00, 5, 5 );
		Component keyboard = new Part("Keyboard", 30.00, 1, 2 );

		computer.add(systemUnit);
		computer.add(monitor);
		computer.add(keyboard);
		
		Component fan = new Part("Fan", 8.00, 3, 1);
		fan.add(new Part("Junk", 1, 2, 3)); // Note that nothing happens here...
		Component hdd = new Part("HDD", 90.00, 5, 1);
		Component cabinet = new Part("Cabinet", 150.00, 10, 20 );
		systemUnit.add(fan);
		systemUnit.add(hdd);
		systemUnit.add(cabinet);

		Component chassis = new Composite("Chassis");
		Component cpu = new Part("CPU", 100.00, 10, 1 );
		Component memory = new Part("Memory", 200.00, 10, 1 );
		Component gpu = new Part("ATI FireGL", 300.00, 10, 1 );
		Component motherboard = new Part("ASUS", 200.00, 10, 1 );
		chassis.add(cpu);
		chassis.add(memory);
		chassis.add(gpu);
		chassis.add(motherboard);

		systemUnit.add(chassis);
		
		double price = systemUnit.getPrice();
		System.out.println("Price = " + price );
//		System.out.println("Health = " + systemUnit.getFaults() );
				
//		Visitor v = new CostVisitor();
//		visitEachComponent(v, systemUnit, 0);
//		System.out.println("Highest cost item is: " + v.getResult() );
	}

	// Recursive method that iterates through the entire hierarchy starting 
	// at a specific node
	// @node - the node to begin at
	// @level - the level of indentation
	public static void visitEachComponent(Visitor v, Component node, int level) {
		System.out.println("Component: " + node  );
		node.accept(v);

		Collection<Component> c = node.getChildren();
		for( Component child: c) {
			visitEachComponent(v, child, level+1);
		}		
	}

	// Recursive method that iterates through the entire hierarchy starting 
	// at a specific node
	// @node - the node to begin at
	// @level - the level of indentation
	public static void listEachComponent(Component node, int level) {
		for( int l=0;l<level; l++)
			System.out.print("    ");
		System.out.println("START: " + node + " price = " + node.getPrice() );

		//TODO: complete this method, avoiding the use of if() or instanceof
		// to determine whether node is a Part or a Composite, and then determine
		// the total weight, cost, or power requirement for the entire system.
		Collection<Component> c = node.getChildren();
		for( Component child: c) {
			listEachComponent(child, level+1);
		}
		
		for( int l=0;l<level; l++)
			System.out.print("    ");
		System.out.println("END: " + node);
	}

}
