package composite;

import java.util.Collection;
import visitors.*;

/**
 * The interface for the Composite Pattern
 */
public abstract class Component {
	// specific methods for the context in which this pattern is applied 
	public abstract double getPrice(); 
	public abstract double getWeight();
	public abstract double getAmps();
	/* package */ abstract int getFaults(); // number of errors encountered
	protected int faults; // errors count


	// the Composite Pattern methods that both Composites and Parts must implement
	/**
	 * adds a Component to the collection
	 * @param c component to be added
	 * @return true if successful; false otherwise
	 */ 
	public abstract boolean add(Component c);

	/**
	 * removes a component from the collection
	 * @param c component to be removed
	 * @return true if successful; false otherwise
	 */
	public abstract boolean remove(Component c);

	/**
	 * retrieves a collection of components contained by this component
	 * @return the collection of children; empty collection if no children
	 */
	public abstract Collection<Component> getChildren();

	/**
	 * Inserts a Visitor into this component
	 * @param v the Visitor to be accepted
	 * @return true if accepted
	 */
	public abstract boolean accept(Visitor v);


//	public class DiagnosticData {
//		protected DiagnosticData(){} // make ctor private
//		public int getFaults() {
//			return Component.this.faults;
//		}
//	}
}

