package composite;

import java.util.*;

import visitors.Visitor;

// This class implements a generic computer part
public class Part extends Component{

	private String name; // name of this Part
	private double price; // price of this Part
	private double amps; // current requirement
	private double weight; // weight
	
	// constructor
	public Part(String name, double price, double amps, double weight ) {
		this.name = name; // name of this part
		this.price = price; 
		this.amps = amps; // how much current it draws
		this.weight = weight; 
		this.faults = 0;
	}

	@Override
	/**
	 * Override of Object.toString()
	 */
	public String toString() {
		return name;
	}

	@Override
	public double getPrice() {
		return price;
	}

	@Override
	public double getAmps() {
		return this.amps;
	}

	@Override
	public double getWeight() {
		return this.weight;
	}

// "Null" implementation of composite methods for Parts...
	public boolean add(Component c) {
		return false;
	}

	@Override
	public Collection<Component> getChildren() {
		return Collections.emptyList();
	}

	@Override
	public boolean remove(Component c) {
		return false;		
	}
	
	@Override
	/*package*/ int getFaults() {
		Random rn = new Random();
		faults += rn.nextInt(3);
		return faults;
	}

	@Override
	public boolean accept(Visitor v) {
		v.visit(this);
		return true;
	}


}
